@@ -2,7 +2,7 @@
 
 __author__ = """Fabien Mathieu"""
 __email__ = 'fabien.mathieu@normalesup.org'
-__version__ = '0.2.0'
+__version__ = '0.2.1'
 
 from stochastic_matching.model import Model
 from stochastic_matching.graphs import Path, Star, Cycle, Codomino, CycleChain, Complete, Pyramid, HyperPaddle, \
@@ -11,4 +11,4 @@ from stochastic_matching.simulator.age_based import FCFM
 from stochastic_matching.simulator.virtual_queue import VQSimulator
 from stochastic_matching.simulator.size_based import LongestSimulator, PrioritySimulator, QueueSizeSimulator, \
     RandomItemSimulator, \
-    RandomNodeSimulator, SemiGreedy
+    RandomNodeSimulator, FilteringGreedy
