@@ -101,6 +101,8 @@ class QueueSizeSimulator(Simulator, ABC):
     Class derived from :class:`~stochastic_matching.simulator.generic.Simulator`
     for greedy simulator based on queue sizes.
     """
+    name = 'generic_queue_size'
+
     def __init__(self, model, selector = 'longest', selector_kwargs=None,
                               **kwargs):
         self.selector = class_converter(selector, Selector)
@@ -288,10 +290,10 @@ class RandomNodeSimulator(QueueSizeSimulator):
     >>> sim = RandomNodeSimulator(triangle, number_events=1000, seed=42, max_queue=10)
     >>> sim.run()
     >>> sim.logs # doctest: +NORMALIZE_WHITESPACE
-    {'trafic': array([125, 162, 213], dtype=uint32),
+    {'trafic': array([125, 162, 213], dtype=uint64),
      'queue_log': array([[838, 104,  41,  13,   3,   1,   0,   0,   0,   0],
                          [796, 119,  53,  22,   8,   2,   0,   0,   0,   0],
-                         [640, 176,  92,  51,  24,   9,   5,   3,   0,   0]], dtype=uint32),
+                         [640, 176,  92,  51,  24,   9,   5,   3,   0,   0]], dtype=uint64),
      'steps_done': 1000}
 
      Sanity check: results are unchanged if the graph is treated as hypergraph.
@@ -300,10 +302,10 @@ class RandomNodeSimulator(QueueSizeSimulator):
     >>> sim = RandomNodeSimulator(triangle, number_events=1000, seed=42, max_queue=10)
     >>> sim.run()
     >>> sim.logs # doctest: +NORMALIZE_WHITESPACE
-    {'trafic': array([125, 162, 213], dtype=uint32),
+    {'trafic': array([125, 162, 213], dtype=uint64),
      'queue_log': array([[838, 104,  41,  13,   3,   1,   0,   0,   0,   0],
                          [796, 119,  53,  22,   8,   2,   0,   0,   0,   0],
-                         [640, 176,  92,  51,  24,   9,   5,   3,   0,   0]], dtype=uint32),
+                         [640, 176,  92,  51,  24,   9,   5,   3,   0,   0]], dtype=uint64),
      'steps_done': 1000}
 
     A ill diamond graph (simulation ends before completion due to drift).
@@ -311,11 +313,11 @@ class RandomNodeSimulator(QueueSizeSimulator):
     >>> sim = RandomNodeSimulator(sm.CycleChain(rates='uniform'), number_events=1000, seed=42, max_queue=10)
     >>> sim.run()
     >>> sim.logs # doctest: +NORMALIZE_WHITESPACE
-    {'trafic': array([ 7, 10,  1,  4,  7], dtype=uint32),
+    {'trafic': array([ 7, 10,  1,  4,  7], dtype=uint64),
     'queue_log': array([[22, 13,  7,  7,  5, 15,  4,  0,  0,  0],
            [73,  0,  0,  0,  0,  0,  0,  0,  0,  0],
            [69,  3,  1,  0,  0,  0,  0,  0,  0,  0],
-           [13, 10, 11,  4,  4,  4,  2,  9, 11,  5]], dtype=uint32),
+           [13, 10, 11,  4,  4,  4,  2,  9, 11,  5]], dtype=uint64),
     'steps_done': 73}
 
     >>> sim.compute_flow()
@@ -326,7 +328,7 @@ class RandomNodeSimulator(QueueSizeSimulator):
     >>> sim = RandomNodeSimulator(sm.HyperPaddle(rates=[1, 1, 1.5, 1, 1.5, 1, 1]), number_events=1000, seed=42, max_queue=25)
     >>> sim.run()
     >>> sim.logs # doctest: +NORMALIZE_WHITESPACE
-    {'trafic': array([26, 21,  4, 25, 34, 10, 16], dtype=uint32),
+    {'trafic': array([26, 21,  4, 25, 34, 10, 16], dtype=uint64),
     'queue_log': array([[ 85,  37,  36,  41,  22,  29,  46,  17,   1,   0,   0,   0,   0,
               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0],
            [275,  32,   7,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
@@ -341,7 +343,7 @@ class RandomNodeSimulator(QueueSizeSimulator):
               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0],
            [268,  23,  16,   6,   1,   0,   0,   0,   0,   0,   0,   0,   0,
               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0]],
-          dtype=uint32),
+          dtype=uint64),
     'steps_done': 314}
 
     Note that you can reset the simulator before starting another run.
@@ -349,7 +351,7 @@ class RandomNodeSimulator(QueueSizeSimulator):
     >>> sim.reset()
     >>> sim.run()
     >>> sim.logs # doctest: +NORMALIZE_WHITESPACE
-    {'trafic': array([26, 21,  4, 25, 34, 10, 16], dtype=uint32),
+    {'trafic': array([26, 21,  4, 25, 34, 10, 16], dtype=uint64),
     'queue_log': array([[ 85,  37,  36,  41,  22,  29,  46,  17,   1,   0,   0,   0,   0,
               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0],
            [275,  32,   7,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
@@ -364,7 +366,7 @@ class RandomNodeSimulator(QueueSizeSimulator):
               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0],
            [268,  23,  16,   6,   1,   0,   0,   0,   0,   0,   0,   0,   0,
               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0]],
-          dtype=uint32),
+          dtype=uint64),
     'steps_done': 314}
 
     You can display the distribution of queue sizes as a ccdf:
@@ -437,10 +439,10 @@ class LongestSimulator(QueueSizeSimulator):
     >>> sim = LongestSimulator(sm.Cycle(rates=[3, 4, 5]), number_events=1000, seed=42, max_queue=10)
     >>> sim.run()
     >>> sim.logs # doctest: +NORMALIZE_WHITESPACE
-    {'trafic': array([125, 162, 213], dtype=uint32),
+    {'trafic': array([125, 162, 213], dtype=uint64),
     'queue_log': array([[838, 104,  41,  13,   3,   1,   0,   0,   0,   0],
        [796, 119,  53,  22,   8,   2,   0,   0,   0,   0],
-       [640, 176,  92,  51,  24,   9,   5,   3,   0,   0]], dtype=uint32),
+       [640, 176,  92,  51,  24,   9,   5,   3,   0,   0]], dtype=uint64),
     'steps_done': 1000}
 
     A non stabilizable diamond (simulation ends before completion due to drift).
@@ -448,11 +450,11 @@ class LongestSimulator(QueueSizeSimulator):
     >>> sim = LongestSimulator(sm.CycleChain(rates='uniform'), number_events=1000, seed=42, max_queue=10)
     >>> sim.run()
     >>> sim.logs # doctest: +NORMALIZE_WHITESPACE
-    {'trafic': array([38, 38,  7, 37, 40], dtype=uint32),
+    {'trafic': array([38, 38,  7, 37, 40], dtype=uint64),
     'queue_log': array([[127,  74,  28,  37,  21,  32,  16,   1,   2,   1],
            [327,   8,   3,   1,   0,   0,   0,   0,   0,   0],
            [322,  12,   4,   1,   0,   0,   0,   0,   0,   0],
-           [ 91,  80,  47,  37,  37,  23,  11,   3,   5,   5]], dtype=uint32),
+           [ 91,  80,  47,  37,  37,  23,  11,   3,   5,   5]], dtype=uint64),
     'steps_done': 339}
 
     A stabilizable candy (but candies are not good for greedy policies).
@@ -460,7 +462,7 @@ class LongestSimulator(QueueSizeSimulator):
     >>> sim = LongestSimulator(sm.HyperPaddle(rates=[1, 1, 1.5, 1, 1.5, 1, 1]), number_events=1000, seed=42, max_queue=25)
     >>> sim.run()
     >>> sim.logs # doctest: +NORMALIZE_WHITESPACE
-    {'trafic': array([24, 17,  2, 23, 33, 12, 13], dtype=uint32),
+    {'trafic': array([24, 17,  2, 23, 33, 12, 13], dtype=uint64),
     'queue_log': array([[ 24,  32,  45,  38,  22,  43,  31,  34,  20,   3,   0,   0,   0,
               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0],
            [291,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
@@ -475,7 +477,7 @@ class LongestSimulator(QueueSizeSimulator):
               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0],
            [232,  33,  16,   4,   6,   1,   0,   0,   0,   0,   0,   0,   0,
               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0]],
-          dtype=uint32),
+          dtype=uint64),
     'steps_done': 292}
     """
     name = "longest_queue"
@@ -536,10 +538,10 @@ class RandomItemSimulator(QueueSizeSimulator):
     >>> sim = RandomItemSimulator(sm.Cycle(rates=[3, 4, 5]), number_events=1000, seed=42, max_queue=10)
     >>> sim.run()
     >>> sim.logs # doctest: +NORMALIZE_WHITESPACE
-    {'trafic': array([125, 162, 213], dtype=uint32),
+    {'trafic': array([125, 162, 213], dtype=uint64),
     'queue_log': array([[838, 104,  41,  13,   3,   1,   0,   0,   0,   0],
        [796, 119,  53,  22,   8,   2,   0,   0,   0,   0],
-       [640, 176,  92,  51,  24,   9,   5,   3,   0,   0]], dtype=uint32),
+       [640, 176,  92,  51,  24,   9,   5,   3,   0,   0]], dtype=uint64),
     'steps_done': 1000}
 
     A ill braess graph (simulation ends before completion due to drift).
@@ -547,11 +549,11 @@ class RandomItemSimulator(QueueSizeSimulator):
     >>> sim = RandomItemSimulator(sm.CycleChain(rates='uniform'), number_events=1000, seed=42, max_queue=10)
     >>> sim.run()
     >>> sim.logs # doctest: +NORMALIZE_WHITESPACE
-    {'trafic': array([12, 11,  4,  8, 10], dtype=uint32),
+    {'trafic': array([12, 11,  4,  8, 10], dtype=uint64),
     'queue_log': array([[39, 13, 10,  6,  3,  8, 14,  8,  3,  1],
            [96,  5,  3,  1,  0,  0,  0,  0,  0,  0],
            [97,  7,  1,  0,  0,  0,  0,  0,  0,  0],
-           [41, 18, 13, 13,  8,  5,  1,  2,  3,  1]], dtype=uint32),
+           [41, 18, 13, 13,  8,  5,  1,  2,  3,  1]], dtype=uint64),
     'steps_done': 105}
 
     A working candy (but candies are not good for greedy policies).
@@ -559,7 +561,7 @@ class RandomItemSimulator(QueueSizeSimulator):
     >>> sim = RandomItemSimulator(sm.HyperPaddle(rates=[1, 1, 1.5, 1, 1.5, 1, 1]), number_events=1000, seed=42, max_queue=25)
     >>> sim.run()
     >>> sim.logs # doctest: +NORMALIZE_WHITESPACE
-    {'trafic': array([83, 62, 36, 58, 75, 48, 74], dtype=uint32),
+    {'trafic': array([83, 62, 36, 58, 75, 48, 74], dtype=uint64),
     'queue_log': array([[537, 135,  65,  62,  34,  20,  25,  30,  48,  12,   4,   0,   0,
               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0],
            [792, 130,  28,  14,   8,   0,   0,   0,   0,   0,   0,   0,   0,
@@ -574,7 +576,7 @@ class RandomItemSimulator(QueueSizeSimulator):
               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0],
            [732, 125,  74,  25,  11,   3,   2,   0,   0,   0,   0,   0,   0,
               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0]],
-          dtype=uint32),
+          dtype=uint64),
     'steps_done': 972}
     """
     name = 'random_item'
@@ -583,7 +585,7 @@ class RandomItemSimulator(QueueSizeSimulator):
         super(RandomItemSimulator, self).__init__(model, selector='random_item', **kwargs)
 
 
-class SemiGreedy(QueueSizeSimulator):
+class FilteringGreedy(QueueSizeSimulator):
     """
     Longest queue simulator where some edges can be forbidden unless some threshold on queue size is reached.
 
@@ -591,8 +593,11 @@ class SemiGreedy(QueueSizeSimulator):
     ----------
     model: :class:`~stochastic_matching.model.Model`
         Model to simulate.
-    forbidden_edges: :class:`list` or :class:`~numpy.ndarray`
+    forbidden_edges: :class:`list` or :class:`~numpy.ndarray`, optional
         Egdes that should not be used.
+    weights: :class:`~numpy.ndarray`, optional
+        Target rewards on edges. If weights are given, the forbidden edges are computed to match the target
+        (overrides forbidden_edges argument).
     threshold: :class:`int`, optional
         Limit on queue size to apply edge interdiction (enforce stability on injective-only vertices).
     **kwargs
@@ -603,7 +608,15 @@ class SemiGreedy(QueueSizeSimulator):
 
     >>> import stochastic_matching as sm
     >>> diamond = sm.CycleChain(rates=[1, 2, 2, 1])
-    >>> diamond.run('semi_greedy', forbidden_edges=[0, 4], seed=42,
+    >>> diamond.run('filtering', forbidden_edges=[0, 4], seed=42,
+    ...                            threshold=100, number_events=1000, max_queue=1000)
+    True
+    >>> diamond.simulation
+    array([0.   , 0.954, 0.966, 0.954, 0.   ])
+
+    Same result can be achieved by putting low weights on 0 and 4.
+
+    >>> diamond.run('filtering', weights=[1, 2, 2, 2, 1], seed=42,
     ...                            threshold=100, number_events=1000, max_queue=1000)
     True
     >>> diamond.simulation
@@ -611,15 +624,50 @@ class SemiGreedy(QueueSizeSimulator):
 
     To compare with the priority-based pure greedy version:
 
-    >>> diamond.run('priority', weights=[0, 1, 1, 1, 0], number_events=1000, max_queue=1000, seed=42)
+    >>> diamond.run('priority', weights=[1, 2, 2, 2, 1], number_events=1000, max_queue=1000, seed=42)
     True
     >>> diamond.simulation
     array([0.444, 0.63 , 0.966, 0.63 , 0.324])
-    """
-    name = 'semi_greedy'
 
-    def __init__(self, model, forbidden_edges=None, threshold=None, **kwargs):
-        super(SemiGreedy, self).__init__(model, **kwargs)
+    Another example with other rates.
+
+    >>> diamond.rates=[4, 5, 2, 1]
+
+    Optimize with the first and last edges that provide less reward.
+
+    >>> diamond.run('filtering', weights=[1, 2, 2, 2, 1], seed=42,
+    ...                            threshold=100, number_events=1000, max_queue=1000)
+    True
+    >>> diamond.simulation
+    array([3.264, 0.888, 0.948, 0.84 , 0.   ])
+
+    Increase the reward on the first edge.
+
+    >>> diamond.run('filtering', weights=[4, 2, 2, 2, 1], seed=42,
+    ...                            threshold=100, number_events=1000, max_queue=1000)
+    True
+    >>> diamond.simulation
+    array([4.152, 0.   , 0.996, 0.   , 0.84 ])
+
+    On bijective graphs, no edge is forbidden whatever the weights.
+
+    >>> paw = sm.Tadpole()
+    >>> paw.run('filtering', weights=[6, 3, 1, 2], seed=42,
+    ...                            threshold=100, number_events=1000, max_queue=1000)
+    True
+    >>> paw.simulation
+    array([1.048, 1.056, 1.016, 0.88 ])
+    """
+    name = 'filtering'
+
+    def __init__(self, model, forbidden_edges=None, threshold=None, weights=None, **kwargs):
+        super(FilteringGreedy, self).__init__(model, **kwargs)
+        if weights is not None:
+            weights = np.array(weights)
+            flow = model.optimize_rates(weights)
+            forbidden_edges = [i for i in range(model.m) if flow[i]==0]
+            if len(forbidden_edges) == 0:
+                forbidden_edges = None
         choicer = SizeChoicer(self.model).yield_jit()
         selector = LongestSelector(self.model).yield_jit()
         self.core = qs_core_maker(choicer, selector, forbidden_edges=forbidden_edges, threshold=threshold)
